==> em_product-0.1.7-py3-none-any.whl: em_product/product.py <==
from pydantic import BaseModel, ValidationError, field_validator
from datetime import datetime
from typing import Optional, List, Dict, Literal, Union, Any


class BaseVariant(BaseModel):
    sku: str
    price: float
    available_qty: Optional[int]


class Variant(BaseVariant):
    barcode: Optional[str]
    variant_id: Optional[str]
    option_values: List[Dict[str, str]]
    images: Optional[str]

    @field_validator("images")
    def validate_images(cls, images):
        if images is None:
            return images

        try:
            for image in images.split(";"):
                if not image.startswith("http"):
                    raise ValueError("images must be a string")
            return images
        except Exception as e:
            raise ValueError(str(e))


class ProductSource(BaseModel):
    _id: str
    date: Any
    url: str
    source: str
    existence: bool
    sku: Optional[str]
    price: Optional[float]
    available_qty: Optional[int]
    images: Optional[str]
    product_id: Optional[Union[str, int]]
    title: Optional[str]
    title_en: Optional[str]
    description: Optional[str]
    description_en: Optional[str]
    summary: Optional[str]
    upc: Optional[str]
    brand: Optional[str]
    specifications: Optional[List[Dict[str, str]]]
    categories: Optional[str]
    videos: Optional[str]
    options: Optional[List[Dict[Literal["name"], str]]]
    variants: Optional[List[Variant]]
    returnable: Optional[bool]
    reviews: Optional[int]
    rating: Optional[float]
    sold_count: Optional[int]
    shipping_fee: Optional[float]
    shipping_days_min: Optional[int]
    shipping_days_max: Optional[int]
    weight: Optional[float]
    width: Optional[float]
    height: Optional[float]
    length: Optional[float]
    has_only_default_variant: Optional[bool]


class StandardProduct(BaseVariant):
    date: str
    url: str
    source: str
    images: str
    product_id: Union[str, int]
    existence: bool
    title: str
    title_en: Optional[str]
    description: Optional[str]
    description_en: Optional[str]
    summary: Optional[str]
    upc: Optional[str]
    brand: Optional[str]
    specifications: Optional[List[Dict[str, str]]]
    categories: Optional[str]
    videos: Optional[str]
    options: Optional[List[Dict[Literal["name"], str]]]
    variants: Optional[List[Variant]]
    returnable: Optional[bool]
    reviews: Optional[int]
    rating: Optional[float]
    sold_count: Optional[int]
    shipping_fee: float
    shipping_days_min: Optional[int]
    shipping_days_max: Optional[int]
    weight: Optional[float]
    width: Optional[float]
    height: Optional[float]
    length: Optional[float]
    has_only_default_variant: Optional[bool]

    @field_validator("date")
    def validate_date(cls, value):
        try:
            parsed_date = datetime.fromisoformat(value)
            format_date = parsed_date.replace(microsecond=0).isoformat()
        except ValueError:
            raise ValueError("date must be in ISO 8601 format and omit microseconds")
        return format_date

    @field_validator("specifications")
    def validate_specifications(cls, specs):
        if specs is None:
            return specs
        try:
            for spec in specs:
                spec_bk = spec.copy()
                spec_bk.pop("name")
                spec_bk.pop("value")
                if spec_bk:
                    raise ValueError(
                        "1. Specifications must be None or List.\n"
                        "2. specification must be like {name:name, value:value}"
                    )
        except Exception as e:
            raise ValueError(str(e))

        return specs

    @field_validator("url")
    def validate_url(cls, url):
        if url is None:
            return url

        try:
            if not url.startswith("https"):
                raise ValueError("url must be a string and startswith http")
        except Exception as e:
            raise ValueError(str(e))

        return url

    @field_validator("options")
    def validate_options(cls, options):
        if options is None:
            return options

        try:
            for option in options:
                option.pop("id", None)
                option.pop("name")
                if option:
                    raise ValueError(
                        "1. Options must be None or List. \n"
                        "2. option must be like {name:name} or {name:name, id:id}"
                    )
        except Exception as e:
            raise ValueError(str(e))

        return options

    @field_validator("images")
    def validate_images(cls, images):
        try:
            for image in images.split(";"):
                if not image.startswith("http"):
                    raise ValueError("images must be a string")
        except Exception as e:
            raise ValueError(str(e))

        return images


if __name__ == "__main__":
    try:
        product = StandardProduct(
            date="2024-06-11T12:34:56",
            url="http://example.com/product/123",
            source="example",
            product_id="123456",
            existence=True,
            title="Example Product",
            title_en="Example Product EN",
            description="This is an example product.",
            description_en="This is an example product.",
            summary="Example summary.",
            sku="SKU12345",
            upc="123456789012",
            brand="Example Brand",
            specifications=[{"key1": "value1"}, {"key2": "value2"}],
            categories="Example Category",
            images="http://example.com/image.jpg",
            videos="http://example.com/video.mp4",
            price=19.99,
            available_qty=100,
            options=[{"name": "Color"}],
            variants=[
                Variant(
                    sku="SKU12345-RED",
                    option_values=[{"name": "Color", "value": "Red"}],
                    price=19.99,
                    available_qty=50,
                    barcode="test",
                    variant_id="test",
                    images="http://example.com/image.jpg",
                )
            ],
            returnable=True,
            reviews=10,
            rating=4.5,
            sold_count=100,
            shipping_fee=5.99,
            shipping_days_min=3,
            shipping_days_max=5,
            weight=0.5,
            width=10.0,
            height=5.0,
            length=15.0,
            has_only_default_variant=False,
        )
        print(product)
    except ValidationError as e:
        print(e)

==> em_product-0.1.8-py3-none-any.whl: em_product/product.py <==
from pydantic import BaseModel, ValidationError, field_validator
from datetime import datetime
from typing import Optional, List, Dict, Literal, Union, Any


class BaseVariant(BaseModel):
    sku: str
    price: float
    available_qty: Optional[int]


class OptionValue(BaseModel):
    option_id: Optional[str]
    option_value_id: Optional[str]
    option_name: str
    option_value: str


class Variant(BaseVariant):
    barcode: Optional[str]
    variant_id: Optional[str]
    option_values: List[OptionValue]
    images: Optional[str]

    @field_validator("images")
    def validate_images(cls, images):
        if images is None:
            return images

        try:
            for image in images.split(";"):
                if not image.startswith("http"):
                    raise ValueError("images must be a string")
            return images
        except Exception as e:
            raise ValueError(str(e))


class ProductSource(BaseModel):
    _id: str
    date: Any
    url: str
    source: str
    existence: bool
    sku: Optional[str]
    price: Optional[float]
    available_qty: Optional[int]
    images: Optional[str]
    product_id: Optional[Union[str, int]]
    title: Optional[str]
    title_en: Optional[str]
    description: Optional[str]
    description_en: Optional[str]
    summary: Optional[str]
    upc: Optional[str]
    brand: Optional[str]
    specifications: Optional[List[Dict[str, str]]]
    categories: Optional[str]
    videos: Optional[str]
    options: Optional[List[Dict[Literal["name"], str]]]
    variants: Optional[List[Variant]]
    returnable: Optional[bool]
    reviews: Optional[int]
    rating: Optional[float]
    sold_count: Optional[int]
    shipping_fee: Optional[float]
    shipping_days_min: Optional[int]
    shipping_days_max: Optional[int]
    weight: Optional[float]
    width: Optional[float]
    height: Optional[float]
    length: Optional[float]
    has_only_default_variant: Optional[bool]


class StandardProduct(BaseVariant):
    date: str
    url: str
    source: str
    images: str
    product_id: Union[str, int]
    existence: bool
    title: str
    title_en: Optional[str]
    description: Optional[str]
    description_en: Optional[str]
    summary: Optional[str]
    upc: Optional[str]
    brand: Optional[str]
    specifications: Optional[List[Dict[str, str]]]
    categories: Optional[str]
    videos: Optional[str]
    options: Optional[List[Dict[Literal["name"], str]]]
    variants: Optional[List[Variant]]
    returnable: Optional[bool]
    reviews: Optional[int]
    rating: Optional[float]
    sold_count: Optional[int]
    shipping_fee: float
    shipping_days_min: Optional[int]
    shipping_days_max: Optional[int]
    weight: Optional[float]
    width: Optional[float]
    height: Optional[float]
    length: Optional[float]
    has_only_default_variant: Optional[bool]

    @field_validator("date")
    def validate_date(cls, value):
        try:
            parsed_date = datetime.fromisoformat(value)
            format_date = parsed_date.replace(microsecond=0).isoformat()
        except ValueError:
            raise ValueError("date must be in ISO 8601 format and omit microseconds")
        return format_date

    @field_validator("specifications")
    def validate_specifications(cls, specs):
        if specs is None:
            return specs
        try:
            for spec in specs:
                spec_bk = spec.copy()
                spec_bk.pop("name")
                spec_bk.pop("value")
                if spec_bk:
                    raise ValueError(
                        "1. Specifications must be None or List.\n"
                        "2. specification must be like {name:name, value:value}"
                    )
        except Exception as e:
            raise ValueError(str(e))

        return specs

    @field_validator("url")
    def validate_url(cls, url):
        if url is None:
            return url

        try:
            if not url.startswith("https"):
                raise ValueError("url must be a string and startswith http")
        except Exception as e:
            raise ValueError(str(e))

        return url

    @field_validator("options")
    def validate_options(cls, options):
        if options is None:
            return options

        try:
            for option in options:
                option.pop("id", None)
                option.pop("name")
                if option:
                    raise ValueError(
                        "1. Options must be None or List. \n"
                        "2. option must be like {name:name} or {name:name, id:id}"
                    )
        except Exception as e:
            raise ValueError(str(e))

        return options

    @field_validator("images")
    def validate_images(cls, images):
        try:
            for image in images.split(";"):
                if not image.startswith("http"):
                    raise ValueError("images must be a string")
        except Exception as e:
            raise ValueError(str(e))

        return images


if __name__ == "__main__":
    try:
        product = StandardProduct(
            date="2024-06-11T12:34:56",
            url="http://example.com/product/123",
            source="example",
            product_id="123456",
            existence=True,
            title="Example Product",
            title_en="Example Product EN",
            description="This is an example product.",
            description_en="This is an example product.",
            summary="Example summary.",
            sku="SKU12345",
            upc="123456789012",
            brand="Example Brand",
            specifications=[{"key1": "value1"}, {"key2": "value2"}],
            categories="Example Category",
            images="http://example.com/image.jpg",
            videos="http://example.com/video.mp4",
            price=19.99,
            available_qty=100,
            options=[{"name": "Color"}],
            variants=[
                Variant(
                    sku="SKU12345-RED",
                    option_values=[{"name": "Color", "value": "Red"}],
                    price=19.99,
                    available_qty=50,
                    barcode="test",
                    variant_id="test",
                    images="http://example.com/image.jpg",
                )
            ],
            returnable=True,
            reviews=10,
            rating=4.5,
            sold_count=100,
            shipping_fee=5.99,
            shipping_days_min=3,
            shipping_days_max=5,
            weight=0.5,
            width=10.0,
            height=5.0,
            length=15.0,
            has_only_default_variant=False,
        )
        print(product)
    except ValidationError as e:
        print(e)
